from fastapi import FastAPI, BackgroundTasks
from fastapi.responses import JSONResponse
import asyncio
import datetime

from utils.publisher import RabbitMQPublisher


app = FastAPI(    title="NY BUS Producer API",
    description="IBD Producer API to publish NY BUS data.",)

publisher = RabbitMQPublisher()

@app.get("/", summary="Root Endpoint")
async def root():
    """
    Root Endpoint
    This endpoint provides a greeting message and directs users to the Swagger UI documentation for the API.
    By navigating to `/docs`, users can access the interactive API documentation where they can see all
    available endpoints, their expected parameters, and try out the API directly from the browser.
    """
    return {"message": "Welcome to the API. Check /docs for Swagger documentation."}

@app.post("/start")
async def start_publishing(background_tasks: BackgroundTasks):
    """Start publishing messages to RabbitMQ"""
    if publisher.is_publishing:
        return JSONResponse(
            status_code=400,
            content={"message": "Publisher is already running"}
        )
    else:
        publisher.connect()

    background_tasks.add_task(publisher.start_publishing)
    return {"message": "Started publishing messages"}


@app.post("/stop")
async def stop_publishing():
    """Stop publishing messages to RabbitMQ"""
    publisher.stop_publishing()
    return {"message": "Stopped publishing messages"}


@app.get("/status")
async def get_status():
    """Get the current status of the publisher"""
    return {
        "is_publishing": publisher.is_publishing,
        "current_position": publisher.current_position,
        "timestamp": datetime.now().isoformat()
    }