import csv
import pika
import os
import json
from typing import List, Dict
from random import randint
import asyncio
from utils.logger import LOG
from datetime import datetime, timedelta

class RabbitMQPublisher:
    def __init__(self):
        self.csv_path = "/code/data/nyBUS_small.csv"
        self.rabbitmq_host = os.environ.get("RABBITMQ_HOST")
        self.rabbitmq_port = int(os.environ.get("RABBITMQ_PORT"))
        self.rabbitmq_user = os.environ.get("RABBITMQ_USER")
        self.rabbitmq_password = os.environ.get("RABBITMQ_PASSWORD")
        self.queue_name = os.environ.get("RABBITMQ_QUEUE")
        self.is_publishing = False
        self.current_position = 0
        self.connection = None
        self.channel = None

    def connect(self):
        credentials = pika.PlainCredentials(self.rabbitmq_user, self.rabbitmq_password)
        parameters = pika.ConnectionParameters(host=self.rabbitmq_host, port=self.rabbitmq_port, credentials=credentials)
        self.connection = pika.BlockingConnection(parameters)
        self.channel = self.connection.channel()

    def close(self):
        if self.connection and not self.connection.is_closed:
            self.connection.close()

    def read_csv_chunk(self, chunk_size: int = 5) -> List[Dict]:
        """Read a chunk of rows from the CSV file"""
        chunk = []
        try:
            with open(self.csv_path, 'r') as file:
                reader = csv.DictReader(file)
                # Skip to current position
                for _ in range(self.current_position):
                    next(reader, None)

                # Read chunk_size rows
                for _ in range(chunk_size):
                    row = next(reader, None)
                    if row is None:
                        self.current_position = 0  # Reset to beginning if EOF
                        break
                    chunk.append(row)
                    self.current_position += 1

        except Exception as e:
            LOG.error(f"Error reading CSV: {str(e)}", exc_info=True)

        return chunk

    def publish_to_rabbitmq(self, message: Dict):
        """Publish messages to RabbitMQ queue"""

        # Format the message
        parsed_message = message
        parsed_message["OriginLocation"] = {
            "lat":float(message["OriginLat"]),
            "lon":float(message["OriginLong"])
        }
        del parsed_message["OriginLat"]
        del parsed_message["OriginLong"]

        parsed_message["DestinationLocation"] = {
            "lat":float(message["DestinationLat"]),
            "lon":float(message["DestinationLong"])
        }

        del parsed_message["DestinationLat"]
        del parsed_message["DestinationLong"]

        parsed_message["VehicleLocation"] = {
            "lat":float(message["VehicleLocation.Latitude"]),
            "lon":float(message["VehicleLocation.Longitude"])
        }

        del parsed_message["VehicleLocation.Latitude"]
        del parsed_message["VehicleLocation.Longitude"]

        if message["ExpectedArrivalTime"]:

            expected_date = datetime.fromisoformat(message["ExpectedArrivalTime"])
            parsed_message["ExpectedArrivalTime"] = expected_date.strftime("%Y-%m-%dT%H:%M:%S")

            if message["ScheduledArrivalTime"]:
                scheduled_arrival = message["ScheduledArrivalTime"]
                hour, minute, second = scheduled_arrival.split(":")
                hour = int(hour)
                minute = int(minute)
                second = int(second)

                scheduled_day = expected_date.day

                if hour >= 24:
                    hour = hour % 24

                if expected_date.hour == 00 and hour < 24:
                    scheduled_day = (expected_date - timedelta(days=1)).day

                scheduled_date = expected_date.replace(hour=int(hour), minute=int(minute), second=int(second), day=scheduled_day)

                parsed_message["ScheduledArrivalTime"] = scheduled_date.strftime("%Y-%m-%dT%H:%M:%S")

        parsed_message = json.dumps(parsed_message)

        try:
            self.channel.basic_publish(exchange='',
                                       routing_key=self.queue_name,
                                       body=parsed_message)

        except Exception as e:
            LOG.error(f"Error publishing to RabbitMQ: {str(e)}", exc_info=True)
            raise

    async def start_publishing(self):
        """Start publishing messages"""
        self.is_publishing = True
        while self.is_publishing:
            try:
                messages = self.read_csv_chunk()
                if messages:
                    for message in messages:
                        self.publish_to_rabbitmq(message)
                        await asyncio.sleep(randint(5, 30))
            except Exception as e:
                LOG.error(f"Error in publishing loop: {str(e)}", exc_info=True)
                await asyncio.sleep(5)

    def stop_publishing(self):
        """Stop publishing messages"""
        self.is_publishing = False
        self.close()